// src/contentstack-live-preview-HOC.ts
import { v4 } from "uuid";
import camelCase from "./utils/camelCase.js";
import LivePreview from "./live-preview.js";
import { userInitData } from "./utils/defaults.js";
import { PublicLogger } from "./utils/public-logger.js";
import { handleWebCompare } from "./compare.js";
import { handlePageTraversal } from "./utils/handlePageTraversal.js";
var ContentstackLivePreview = {
  livePreview: null,
  userConfig: null,
  subscribers: {},
  configs: {
    params: {}
  },
  init(userConfig = userInitData) {
    if (typeof window !== "undefined") {
      if (ContentstackLivePreview.livePreview) {
        PublicLogger.warn(
          "You have already initialized the Live Preview SDK. So, any subsequent initialization returns the existing SDK instance."
        );
        return Promise.resolve(ContentstackLivePreview.livePreview);
      } else {
        handleWebCompare();
        handlePageTraversal();
        ContentstackLivePreview.livePreview = new LivePreview(
          userConfig
        );
        ContentstackLivePreview.livePreview.setOnChangeCallback(
          ContentstackLivePreview.publish
        );
        ContentstackLivePreview.livePreview.setConfigFromParams(
          ContentstackLivePreview.configs.params
        );
        ContentstackLivePreview.configs.params = {};
        return Promise.resolve(ContentstackLivePreview.livePreview);
      }
    } else {
      ContentstackLivePreview.userConfig = userConfig;
    }
  },
  get hash() {
    if (!ContentstackLivePreview.livePreview) {
      const urlParams = new URLSearchParams(
        ContentstackLivePreview.configs.params
      );
      return urlParams.get("live_preview") ?? "";
    }
    return ContentstackLivePreview.livePreview.hash;
  },
  /**
   * Sets the live preview hash from the query param which is
   * accessible via `hash` property.
   * @param params query param in an object form
   */
  setConfigFromParams(params = {}) {
    if (!ContentstackLivePreview.livePreview) {
      ContentstackLivePreview.configs.params = params;
      return;
    }
    ContentstackLivePreview.livePreview.setConfigFromParams(params);
  },
  publish() {
    Object.values(
      ContentstackLivePreview.subscribers
    ).forEach((func) => {
      func();
    });
  },
  subscribe(callback) {
    const callbackUid = v4();
    ContentstackLivePreview.subscribers[callbackUid] = callback;
    return callbackUid;
  },
  /**
   * @type {function}
   * @param onChangeCallback A function param to fetch the data from contentstack database
   * @param config An optional object param, pass {skipInitialRender:Boolean} to skip init call to onChangeCallback
   * @returns Subscribed Callback UID
   */
  onEntryChange(onChangeCallback, config) {
    const { skipInitialRender = false } = config || {};
    if (ContentstackLivePreview.userConfig) {
      ContentstackLivePreview.livePreview = new LivePreview(
        ContentstackLivePreview.userConfig
      );
      ContentstackLivePreview.livePreview.setOnChangeCallback(
        ContentstackLivePreview.publish
      );
      ContentstackLivePreview.livePreview.setConfigFromParams(
        ContentstackLivePreview.configs.params
      );
      ContentstackLivePreview.configs.params = {};
      ContentstackLivePreview.userConfig = null;
    }
    const callbackUid = ContentstackLivePreview.subscribe(onChangeCallback);
    if (!skipInitialRender) {
      onChangeCallback();
    }
    return callbackUid;
  },
  /**
   * @type {function}
   * @param onChangeCallback A function param to fetch the data from contentstack database on content change.
   * @returns Subscribed Callback UID
   */
  onLiveEdit(onChangeCallback) {
    return ContentstackLivePreview.onEntryChange(onChangeCallback, {
      skipInitialRender: true
    });
  },
  unsubscribeOnEntryChange(callback) {
    if (typeof callback === "string") {
      if (!ContentstackLivePreview.subscribers[callback]) {
        PublicLogger.warn("No subscriber found with the given id.");
      }
      delete ContentstackLivePreview.subscribers[callback];
    } else if (typeof callback === "function") {
      const isCallbackDeleted = Object.entries(
        ContentstackLivePreview.subscribers
      ).some(([uid, func]) => {
        if (func === callback) {
          delete ContentstackLivePreview.subscribers[uid];
          return true;
        }
        return false;
      });
      if (!isCallbackDeleted) {
        PublicLogger.warn(
          "No subscriber found with the given callback."
        );
      }
    }
  },
  async getGatsbyDataFormat(sdkQuery, prefix) {
    if (typeof sdkQuery.find === "function") {
      return sdkQuery.toJSON().find().then((result) => {
        return result.map((res) => {
          return res.map((entry) => {
            const dataTitle = camelCase(
              `${prefix}_${sdkQuery.content_type_uid}`
            );
            return { [dataTitle]: entry };
          });
        });
      }).catch((err) => {
        console.error(err);
      });
    } else if (typeof sdkQuery.fetch === "function") {
      return sdkQuery.toJSON().fetch().then((ent) => {
        const dataTitle = camelCase(
          `${prefix}_${sdkQuery.content_type_uid}`
        );
        const entry = { [dataTitle]: ent };
        return entry;
      }).catch((err) => {
        console.error(err);
      });
    }
  },
  getSdkVersion() {
    return "2.0.0";
  }
};
var contentstack_live_preview_HOC_default = ContentstackLivePreview;
export {
  contentstack_live_preview_HOC_default as default
};
//# sourceMappingURL=contentstack-live-preview-HOC.js.map